//+------------------------------------------------------------------+
//|                                                     K_Orders.mqh |
//|                                                   Karlis Balcers |
//|                                              http://www.mql5.com |
//+------------------------------------------------------------------+
#property copyright "Karlis Balcers"
#property link      "http://www.mql5.com"

class Order
{
   private:
      MqlTradeRequest trReq;
      MqlTradeResult trRez;
      MqlTradeCheckResult m_checkRex;//to check the results from OrderCheck();
      int iSL;//integer stop loss
      int iTP;
      double dSL;
      double dTP;
      
      double m_partOfLot;//scale factor to trade with from the allowed e.g. 0.2 or 20 percent
      
      double GetSL(bool forBUY)
      {
         if(iSL>0)
         {
            if(forBUY)
            {
               return SymbolInfoDouble(trReq.symbol,SYMBOL_ASK)-iSL*_Point;
            }
            else
            {
               return SymbolInfoDouble(trReq.symbol,SYMBOL_BID)+iSL*_Point;
            }
         }
         else return dSL;
      }
      
      double GetTP(bool forBUY)
      {
         if(iTP>0)
         {
            if(forBUY)
            {
               return SymbolInfoDouble(trReq.symbol,SYMBOL_ASK)+iTP*_Point;
            }
            else
            {
               return SymbolInfoDouble(trReq.symbol,SYMBOL_ASK)-iTP*_Point;
            }
         }
         else return dTP;
      }
      
      void VerifySLTP(bool buy)
      {
         double minSapce=SymbolInfoInteger(trReq.symbol, SYMBOL_TRADE_STOPS_LEVEL) * _Point;
         
         if(buy)
         {
            if(trReq.sl!=0)
               if(SymbolInfoDouble(trReq.symbol,SYMBOL_BID) - trReq.sl < minSapce)
                  trReq.sl = SymbolInfoDouble(trReq.symbol,SYMBOL_BID) - minSapce;
            
            if(trReq.tp!=0)
               if(trReq.tp - trReq.price < minSapce)
                  trReq.tp = trReq.price + minSapce;
         }
         else
         {
            if(trReq.sl!=0)
               if(trReq.sl - SymbolInfoDouble(trReq.symbol,SYMBOL_ASK) < minSapce)
                  trReq.sl = SymbolInfoDouble(trReq.symbol,SYMBOL_ASK) + minSapce;
                  
            if(trReq.tp!=0)
               if(trReq.price - trReq.tp  < minSapce)
                  trReq.tp = trReq.price - minSapce;
         }
      }
      
   public:
   
      Order()
      {
         trReq.action=TRADE_ACTION_DEAL;
         trReq.magic=0;
         trReq.symbol=Symbol();
         trReq.volume=1;
         trReq.price=0;
         trReq.sl=0;               // Stop Loss level of the order
         trReq.tp=0;               // Take Profit level of the order
         trReq.deviation=5;        // Maximal possible deviation from the requested price
         trReq.type=0;
         trReq.type_filling=ORDER_FILLING_AON;
         trReq.type_time=ORDER_TIME_GTC;
         trReq.expiration=0;
         trReq.comment="";
         iSL=0;
         iTP=0;
         dSL=0;
         dTP=0;
         m_partOfLot=0.2;
      }
      
      bool Buy()
      {
         double required_margin=0.0;
         double minVOlum=SymbolInfoDouble(trReq.symbol,SYMBOL_VOLUME_MIN);
         trReq.type=ORDER_TYPE_BUY;
         trReq.price=SymbolInfoDouble(trReq.symbol,SYMBOL_ASK);
         trReq.sl=GetSL(true);
         trReq.tp=GetTP(true);
     if(OrderCalcMargin(trReq.type,trReq.symbol,1.0,trReq.price,required_margin))  // Calculate margin required on 1 lot
         {
            trReq.volume = NormalizeDouble(AccountInfoDouble(ACCOUNT_FREEMARGIN)/required_margin*m_partOfLot,2); // Maximal lots that could be opened with free margin    
            if(trReq.volume<minVOlum) trReq.volume = minVOlum;
         }
         if(trReq.volume) {
        //trReq.volume=SymbolInfoDouble(trReq.symbol,SYMBOL_VOLUME_MIN);
         //{         
          //trReq.volume=NormalizeDouble(trReq.volume/m_stdLot,4);
        Print("SYMBOL_VOLUME_MIN:", SymbolInfoDouble(trReq.symbol,SYMBOL_VOLUME_MIN));
         Print("SYMBOL_VOLUME_MAX:", SymbolInfoDouble(trReq.symbol,SYMBOL_VOLUME_MAX));
         Print("SYMBOL_VOLUME_STEP:", SymbolInfoDouble(trReq.symbol,SYMBOL_VOLUME_STEP));
          Print("SYMBOL_VOLUME_LIMIT:", SymbolInfoDouble(trReq.symbol,SYMBOL_VOLUME_LIMIT));
         Print("ASK price:", trReq.price);
         printf("ACCOUNT_EQUITY =  %G",AccountInfoDouble(ACCOUNT_EQUITY));
         printf("ACCOUNT_FREEMARGIN =  %G",AccountInfoDouble(ACCOUNT_FREEMARGIN));
         Print("Lots to buy: ",trReq.volume);
        }else  Print("Problem to buy: ",trReq.volume);
         VerifySLTP(true);
               
         return Execute();
      }
      
      bool Sell()
      {
         double required_margin=0.0;
         double minVOlum=SymbolInfoDouble(trReq.symbol,SYMBOL_VOLUME_MIN);
         trReq.type=ORDER_TYPE_SELL;
         trReq.price=SymbolInfoDouble(trReq.symbol,SYMBOL_BID);
         trReq.sl=GetSL(false);
         trReq.tp=GetTP(false);

        //trReq.volume=SymbolInfoDouble(trReq.symbol,SYMBOL_VOLUME_MIN);
     if(OrderCalcMargin(trReq.type,trReq.symbol,1.0,trReq.price,required_margin))  // Calculate margin required on 1 lot
         {
            trReq.volume = NormalizeDouble(AccountInfoDouble(ACCOUNT_FREEMARGIN)/required_margin*m_partOfLot,2); // Maximal lots that could be opened with free margin    
            if(trReq.volume<minVOlum) trReq.volume = minVOlum;
         }
         if(trReq.volume) {         
         //trReq.volume=NormalizeDouble(trReq.volume/m_stdLot,4);
         Print("SYMBOL_VOLUME_MIN:", SymbolInfoDouble(trReq.symbol,SYMBOL_VOLUME_MIN));
         Print("SYMBOL_VOLUME_MAX:", SymbolInfoDouble(trReq.symbol,SYMBOL_VOLUME_MAX));
         Print("SYMBOL_VOLUME_STEP:", SymbolInfoDouble(trReq.symbol,SYMBOL_VOLUME_STEP));
          Print("SYMBOL_VOLUME_LIMIT:", SymbolInfoDouble(trReq.symbol,SYMBOL_VOLUME_LIMIT));
         Print("BID price:", trReq.price);
         printf("ACCOUNT_EQUITY =  %G",AccountInfoDouble(ACCOUNT_EQUITY));
         printf("ACCOUNT_FREEMARGIN =  %G",AccountInfoDouble(ACCOUNT_FREEMARGIN));
         Print("Lots to sell: ",trReq.volume);
        }else Print("Problem Sell: ",trReq.volume);
         VerifySLTP(false);
               
         return Execute();
      }
     
      
      bool Close()
      {
         if(PositionSelect(trReq.symbol))
         {
            if((ENUM_POSITION_TYPE)PositionGetInteger(POSITION_TYPE)==POSITION_TYPE_BUY)
            {
               trReq.type = ORDER_TYPE_SELL;
               trReq.price=SymbolInfoDouble(trReq.symbol,SYMBOL_BID);
            }
            else
            {
               trReq.type = ORDER_TYPE_BUY;
               trReq.price=SymbolInfoDouble(trReq.symbol,SYMBOL_ASK);
            }
            double vol=trReq.volume;
            trReq.sl=0;
            trReq.tp=0;
            trReq.volume = PositionGetDouble(POSITION_VOLUME);
            bool ret=Execute();
            trReq.volume=vol;
            
            return ret;
         }
         else
         {
            Print("Position can not be selected.");
         }
         return true; 
      }
      
      bool Modify()
      {
         if(PositionSelect(trReq.symbol))
         {
            trReq.action=TRADE_ACTION_SLTP;
            if((ENUM_POSITION_TYPE)PositionGetInteger(POSITION_TYPE)==POSITION_TYPE_BUY)
            {
               trReq.sl=GetSL(true);
               trReq.tp=GetTP(true);
               VerifySLTP(true);
            }
            else
            {
               trReq.sl=GetSL(false);
               trReq.tp=GetTP(false);
               VerifySLTP(false);
            }
            if(PositionGetDouble(POSITION_SL)==trReq.sl && PositionGetDouble(POSITION_TP)==trReq.tp)
               return true;
            bool retval=Execute();
            trReq.action=TRADE_ACTION_DEAL;
            return retval;
         }
         else
         {
            Print("Modify()| Position can not be selected.");
         }
         return false;
      }

      bool Close_Volume(double volume)
      {
         if(PositionSelect(trReq.symbol))
         {
            if((ENUM_POSITION_TYPE)PositionGetInteger(POSITION_TYPE)==POSITION_TYPE_BUY)
            {
               trReq.type = ORDER_TYPE_SELL;
               trReq.price=SymbolInfoDouble(trReq.symbol,SYMBOL_BID);
            }
            else
            {
               trReq.type = ORDER_TYPE_BUY;
               trReq.price=SymbolInfoDouble(trReq.symbol,SYMBOL_ASK);
            }
            double vol=trReq.volume;
            trReq.sl=0;
            trReq.tp=0;
            trReq.volume = volume;
            bool ret=Execute();
            trReq.volume=vol;
            
            return ret;
         }
         else
         {
            Print("Position can not be selected.");
         }
         return true; 
      }
      
      bool Execute()
      {
         //---check if there are enough funds:
         if(!OrderCheck(trReq,m_checkRex)){
         Print("Execution of new order failed! Not enough funds! Error = ",GetLastError());
         //--- tune the lots to get the correct trading
         ExpertRemove();//remove the expert to save time and power
         Print(TimeCurrent(),": ",__FUNCTION__," expert advisor will be unloaded");
         return false;
         }
         
         bool err=OrderSend(trReq,trRez);
         if(!err)
         {
            Print("Execution of new order failed! Error = ",GetLastError()," SL=",trReq.sl," TP=",trReq.tp," Bid=", SymbolInfoDouble(Symbol(),SYMBOL_BID)," ASK=", SymbolInfoDouble(Symbol(),SYMBOL_ASK));//, " SWAP_LONG=", SymbolInfoInteger(trReq.symbol, SYMBOL_TRADE_STOPS_LEVEL));
            return false;
         }
         return true;
      }
      
      //==============
      
      void SetMagic(int magic)
      {
         trReq.magic=magic;
      }
      
      void SetComment(string comment)
      {
         trReq.comment=comment;
      }
      
      void SetLots(double lots)
      {
         trReq.volume=lots;
      }
      
      void SetSL_inPrice(double sl_price)
      {
         iSL=0;
         dSL=sl_price;
      }
      
      void SetTP_inPrice(double tp_price)
      {
         iTP=0;
         dTP=tp_price;
      }
      
      void SetSL_inPoints(int sl)
      {
         dSL=0;
         iSL=sl;
      }
      
      void SetTP_inPoints(int tp)
      {
         dTP=0;
         iTP=tp;
      }
      

   
};



 